#include "PrzetwarzanieObrazow.h"

PrzetwarzanieObrazow::PrzetwarzanieObrazow() {
	wartoscGraniczna = WARTOSC_GRANICZNA;
	kolor = Vec3b(0,0,255);
}

Point PrzetwarzanieObrazow:: polozeniePedzla (Mat obraz) {
	Mat piksele = wybierzPiksele(obraz);
	Kontur wybranyKontur = filtruj(piksele);
	return wyznaczPolozenie(wybranyKontur);
}

Mat PrzetwarzanieObrazow::wybierzPiksele(Mat obraz) {
	Vec3b szukanaBarwa = wyliczBarwe(kolor);
	obraz.copyTo(_wybierzPikseleBarwa);
	konwertujNaBarwe(_wybierzPikseleBarwa);
	//imshow("Barwa", _wybierzPikseleBarwa);
	absdiff(_wybierzPikseleBarwa,szukanaBarwa,_wybierzPikseleRoznica);
	_wybierzPikseleWynik = Mat(_wybierzPikseleRoznica.size(),CV_8U);
	for(int i=0;i<_wybierzPikseleWynik.rows;i++) for(int i2=0;i2<_wybierzPikseleWynik.cols;i2++) {
		Vec3b piksel = _wybierzPikseleRoznica.at<Vec3b>(i,i2);
		_wybierzPikseleWynik.at<uchar>(i,i2) = saturate_cast<uchar> ((int)piksel.val[0]+piksel.val[1]+piksel.val[2]);
	}
	threshold(_wybierzPikseleWynik,_wybierzPikseleWynik,wartoscGraniczna,255,THRESH_BINARY_INV);
	//imshow("Wybrane",_wybierzPikseleWynik);
	return _wybierzPikseleWynik;
}

Kontur PrzetwarzanieObrazow::filtruj(Mat wybrane) { 
	vector<Kontur> filtrujKontury;
	findContours(wybrane,filtrujKontury,CV_RETR_EXTERNAL,CV_CHAIN_APPROX_SIMPLE);
	double najwiekszyRozmiar = 0;
    int nrNajwiekszego;
    for(int i=0;i<filtrujKontury.size();i++) {
        double rozmiar = abs(contourArea(filtrujKontury[i]));
        if(rozmiar>najwiekszyRozmiar) {
            najwiekszyRozmiar = rozmiar;
            nrNajwiekszego = i;
        }
    }
    return najwiekszyRozmiar > 0 ? filtrujKontury[nrNajwiekszego] : Kontur();
}

Point PrzetwarzanieObrazow::wyznaczPolozenie(Kontur kontur) {
    if(kontur.size()==0 || abs(contourArea(kontur)) < MINIMALNY_ROZMIAR_KONTURU) {
        return Point();
    }
    else {
        Moments momenty = moments(kontur);
        return Point(momenty.m10/momenty.m00,momenty.m01/momenty.m00);
    }
}


Vec3b PrzetwarzanieObrazow::wyliczBarwe(Vec3b piksel) {
	int suma = (int)piksel.val[0]+piksel.val[1]+piksel.val[2];
	if(suma!=0) { 
		piksel.val[0] = piksel.val[0] * 255 / suma;
		piksel.val[1] = piksel.val[1] * 255 / suma;
		piksel.val[2] = 255 - piksel.val[0] - piksel.val[1];
	}
	return piksel;
}

void PrzetwarzanieObrazow::konwertujNaBarwe(Mat &obraz) {
	for(int i=0;i<obraz.rows;i++) for(int i2=0;i2<obraz.cols;i2++) {
		obraz.at<Vec3b>(i,i2) = wyliczBarwe(obraz.at<Vec3b>(i,i2));
	}
}

void PrzetwarzanieObrazow::wyliczDobreParametry(Mat probka, Vec3b& kolor, int& tolerancja) {
	Mat probkaBarwa;
	probka.copyTo(probkaBarwa);
	konwertujNaBarwe(probkaBarwa);
	Scalar suma = sum(probkaBarwa);
	kolor = Vec3b(suma.val[0] / probkaBarwa.total(), suma.val[1] / probkaBarwa.total(), suma.val[2] / probkaBarwa.total());
	kolor = wyliczBarwe(kolor);
	vector<int> listaRoznic;
	for(int i=0;i<probkaBarwa.rows;i++) for(int i2=0;i2<probkaBarwa.cols;i2++) {
		Vec3b wartosc = probkaBarwa.at<Vec3b>(i,i2);
		listaRoznic.push_back(abs(wartosc[0] - kolor[0]) + abs(wartosc[1] - kolor[1]) + abs(wartosc[2] - kolor[2]));
	}
	sort(listaRoznic.begin(),listaRoznic.end());
	tolerancja = min(listaRoznic[8*listaRoznic.size()/10],70);
}