#include <cstdio>
using namespace std;

#include "opencv2/highgui/highgui.hpp"
using namespace cv;

#include "stany.h"

#ifdef _WIN32 
	#include <windows.h>
#elif defined  __unix__
	
#endif
int screenX, screenY;
const int ROZMIAR_OKNA_KALIBRACJI = 50;

VideoCapture kamera;
StanGry stanGry;
Mat aktualnyObraz;
Mat wyswietlanyObraz;

Mat poprawionyWyswietlanyObraz;
void wyswietlZachowujacProporcje(Mat obrazek) {
	double dodatek = (double)screenX/screenY*obrazek.rows - obrazek.cols;
	
	poprawionyWyswietlanyObraz = Mat(obrazek.rows,obrazek.cols+dodatek,obrazek.type(),Scalar_<uchar>(0));
	Mat kawalek = Mat(poprawionyWyswietlanyObraz,Range(0,obrazek.rows),Range(dodatek/2,dodatek/2+obrazek.cols));
	obrazek.copyTo(kawalek);
	imshow("Maluj plot!",poprawionyWyswietlanyObraz);
}

void kalibrujPedzelGracza(StanGracza &gracz) {
	kamera >> aktualnyObraz;
	flip(aktualnyObraz,aktualnyObraz,1);
	aktualnyObraz.copyTo(wyswietlanyObraz);
	Uzytki::wypiszTekst(wyswietlanyObraz, gracz.nazwa + "! Wybierz pedzel!",Point(0,25));
	wyswietlZachowujacProporcje(wyswietlanyObraz);
	waitKey(3000);
	
	while(waitKey(100)!=' ') {
		kamera >> aktualnyObraz;
		flip(aktualnyObraz,aktualnyObraz,1);
		aktualnyObraz.copyTo(wyswietlanyObraz);
		
		Rect polozenieOkna = Rect(aktualnyObraz.cols/2-ROZMIAR_OKNA_KALIBRACJI,aktualnyObraz.rows/2-ROZMIAR_OKNA_KALIBRACJI,ROZMIAR_OKNA_KALIBRACJI,ROZMIAR_OKNA_KALIBRACJI);
		Mat okno = Mat(aktualnyObraz,polozenieOkna);
		gracz.ustawParametryPedzla(okno);
		
		Mat pikselePedzla = gracz.stanMalowania.przetwarzanieObrazow.wybierzPiksele(aktualnyObraz);
		wyswietlanyObraz.setTo(Scalar(255,0,0),pikselePedzla);
		rectangle(wyswietlanyObraz,polozenieOkna,Scalar(0,255,0),3);
		wyswietlZachowujacProporcje(wyswietlanyObraz);
		
		int liczbaPikseliPedzlaOkno = countNonZero(Mat(pikselePedzla,polozenieOkna));
		int liczbaPikseliPedzla = countNonZero(pikselePedzla);
		if(2*liczbaPikseliPedzlaOkno > liczbaPikseliPedzla) {
			break;
		}
	}
}

int main(int argc, char * argv[]) {

	if(imread("plot.png").data == NULL) {
		printf("Brak pliku plot.png z zarysem plotu!\n"); 
		getchar();
		return -1;
	}

	kamera.open(0);
	if(!kamera.isOpened()){
		printf("Nie udalo sie zawiazac polaczenia z kamera!\n"); 
		getchar();
	}
	else {
		printf("Kamera podlaczona!\n");
		int liczbaGraczy;
		printf("Podaj liczbe graczy (1 lub 2): ");
		scanf("%d",&liczbaGraczy);
		if(liczbaGraczy!=1 && liczbaGraczy !=2) {
			printf("Podano nieprawidlowa liczbe graczy (tylko 1 albo 2).");
			getchar();
			return -1;
		}

		char imionaGraczy[2][100];
		for(int i=0;i<liczbaGraczy;i++) {
			printf("Podaj imie gracza %d: ",i+1);
			scanf("%s",imionaGraczy[i]);
			stanGry.gracze[i].nazwa = imionaGraczy[i];
		}		

		#ifdef _WIN32 
			screenX = GetSystemMetrics(SM_CXSCREEN);
			screenY = GetSystemMetrics(SM_CYSCREEN);
		#elif defined  __unix__
			screenX = 640;
			screenY = 480;
		#endif
		
		cvNamedWindow("Maluj plot!", CV_WINDOW_NORMAL);
		cvMoveWindow("Maluj plot!", -5, -5);
		cvResizeWindow("Maluj plot!", screenX, screenY);
		
		kalibrujPedzelGracza(stanGry.gracze[0]);
		if(liczbaGraczy==2) kalibrujPedzelGracza(stanGry.gracze[1]);

		kamera >> aktualnyObraz;
		flip(aktualnyObraz,aktualnyObraz,1);
		aktualnyObraz.copyTo(wyswietlanyObraz);
		Uzytki::wypiszTekst(wyswietlanyObraz,"Przygotuj sie do gry!",Point(0,25));
		wyswietlZachowujacProporcje(wyswietlanyObraz);
		waitKey(3000);

		stanGry.rozpocznijGre(20);

		while(waitKey(10)!=' ') {
			kamera >> aktualnyObraz;
			flip(aktualnyObraz,aktualnyObraz,1);
			stanGry.uaktualnijStanGry(aktualnyObraz);
			aktualnyObraz.copyTo(wyswietlanyObraz);
			stanGry.wyswietlStanGry(wyswietlanyObraz);
			wyswietlZachowujacProporcje(wyswietlanyObraz);
			//imshow("Maluj plot!",wyswietlanyObraz);
			if(stanGry.graTrwa() == false) break;
		}

		aktualnyObraz.copyTo(wyswietlanyObraz);
		Uzytki::wypiszTekst(wyswietlanyObraz,"Koniec gry!",Point(0,25));
		wyswietlZachowujacProporcje(wyswietlanyObraz);
		waitKey(3000);

		wyswietlanyObraz.setTo(Vec3b(0,0,0));
		stanGry.wyswietlNajlepszych(wyswietlanyObraz);
		wyswietlZachowujacProporcje(wyswietlanyObraz);
		waitKey(0);
	}
	return 0;
}
